import { BorderStyle, FillPatterns, Workbook, Worksheet } from 'exceljs';

export type Arrayable<T> = T | T[];
export type ExportContextType = Arrayable<ExportParamsType>;
export type ExportParamsType = Omit<ExportOptions, keyof ContextAttribute>;
export type CommonItemType = StyleOptions & FormatOptions;
export interface FormatterFunOpt {
    key: any;
    row: any;
    value: any;
    rowIndex: number;
    colIndex: number;
}
export interface FormatOptions {
    field?: string;
    formatType?: string;
    formatter?: (value: any) => any;
    render?: (opt: FormatterFunOpt) => any;
}
export interface StyleOptions {
    color?: string;
    width?: number;
    height?: number;
    size?: number;
    align?: 'center' | 'left' | 'right';
    bold?: boolean;
    fllType?: string;
    fillPattern?: FillPatterns;
    fillColor?: string;
    showBorder?: boolean;
    borderType?: BorderStyle;
    borderColor?: string;
}
export interface CellItem extends CommonItemType {
    rowIndex?: number;
    colIndex?: number;
    isMerge?: boolean;
    rowEndIndex?: number;
    colEndIndex?: number;
    value?: string | number;
}
export interface ColumnItem extends CommonItemType {
    title?: string;
    children?: Array<ColumnItem>;
    headerAlign?: 'center' | 'left' | 'right';
}
export interface HeaderOptions extends CommonItemType {
    [key: string]: any;
}
export interface BodyOptions extends CommonItemType {
    headerRowHeight?: number;
    bodyRowHeight?: number;
}
export interface SideOptions extends CommonItemType {
    position?: 'left' | 'right';
}
export interface FooterOptions extends CommonItemType {
    [key: string]: any;
}
export interface ContextInformation {
    columnleafNodeTotal: number;
}
export interface ContextAttribute {
    excelJs: Workbook;
    sheet: Worksheet;
    context: ContextInformation;
}
export interface ExportOptions extends ContextAttribute {
    name?: string;
    header?: HeaderOptions;
    side?: SideOptions;
    body?: BodyOptions;
    footer?: FooterOptions;
    columns?: Array<ColumnItem>;
    cells?: Arrayable<CellItem>;
    data?: Array<any>;
    headerType?: string;
    bodyType?: string;
    sideType?: string;
    footerType?: string;
    tableType?: string;
    beforeExport?: (options: ExportOptions) => any;
}
